chiark / gitweb /
udev: clear lists if a new value is assigned
[elogind.git] / udev.c
diff --git a/udev.c b/udev.c
index 802e41d14719aeea9b3ea858cb4c8977fb379f16..999d25d4e5351ad3c1ffc845f4f8a36418680e39 100644 (file)
--- a/udev.c
+++ b/udev.c
 #include <unistd.h>
 
 #include "libsysfs/sysfs/libsysfs.h"
+#include "udev_libc_wrapper.h"
 #include "udev.h"
 #include "udev_utils.h"
 #include "udev_sysfs.h"
 #include "udev_version.h"
-#include "namedev.h"
+#include "udev_rules.h"
 #include "logging.h"
 
-
-#ifdef LOG
-void log_message(int level, const char *format, ...)
+#ifdef USE_LOG
+void log_message(int priority, const char *format, ...)
 {
        va_list args;
 
-       if (!udev_log)
+       if (priority > udev_log_priority)
                return;
 
        va_start(args, format);
-       vsyslog(level, format, args);
+       vsyslog(priority, format, args);
        va_end(args);
 }
 #endif
 
/* decide if we should manage the whole hotplug event
* for now look if the kernel calls udevsend instead of /sbin/hotplug
  */
 static int manage_hotplug_event(void) {
        char helper[256];
@@ -70,13 +72,15 @@ static int manage_hotplug_event(void) {
        if (fd < 0)
                return 0;
 
-       len = read(fd, helper, 256);
+       len = read(fd, helper, sizeof(helper)-1);
        close(fd);
 
        if (len < 0)
                return 0;
        helper[len] = '\0';
 
+       if (helper[0] == '\0' || helper[0] == '\n')
+               return 1;
        if (strstr(helper, "udevsend"))
                return 1;
 
@@ -99,7 +103,7 @@ int main(int argc, char *argv[], char *envp[])
        struct sysfs_class_device *class_dev;
        struct sysfs_device *devices_dev;
        struct udevice udev;
-       char path[SYSFS_PATH_MAX];
+       char path[PATH_SIZE];
        const char *error;
        const char *action;
        const char *devpath;
@@ -114,13 +118,12 @@ int main(int argc, char *argv[], char *envp[])
        }
 
        logging_init("udev");
-       dbg("version %s", UDEV_VERSION);
-
        udev_init_config();
+       dbg("version %s", UDEV_VERSION);
 
        /* set signal handlers */
        memset(&act, 0x00, sizeof(act));
-       act.sa_handler = (void (*) (int))sig_handler;
+       act.sa_handler = (void (*)(int)) sig_handler;
        sigemptyset (&act.sa_mask);
        act.sa_flags = 0;
        sigaction(SIGALRM, &act, NULL);
@@ -130,17 +133,6 @@ int main(int argc, char *argv[], char *envp[])
        /* trigger timeout to prevent hanging processes */
        alarm(ALARM_TIMEOUT);
 
-       if (strstr(argv[0], "udevstart") || (argc == 2 && strstr(argv[1], "udevstart"))) {
-               dbg("udevstart");
-
-               /* disable all logging, as it's much too slow on some facilities */
-               udev_log = 0;
-
-               namedev_init();
-               retval = udev_start();
-               goto exit;
-       }
-
        /* let the executed programs know if we handle the whole hotplug event */
        managed_event = manage_hotplug_event();
        if (managed_event)
@@ -153,41 +145,45 @@ int main(int argc, char *argv[], char *envp[])
        if (!subsystem && argc == 2)
                subsystem = argv[1];
 
-       udev_init_device(&udev, devpath, subsystem);
+       udev_init_device(&udev, devpath, subsystem, action);
 
        if (!action || !subsystem || !devpath) {
-               dbg("action, subsystem or devpath missing");
+               err("action, subsystem or devpath missing");
                goto hotplug;
        }
 
        /* export logging flag, as called scripts may want to do the same as udev */
-       if (udev_log)
-               setenv("UDEV_LOG", "1", 1);
+       if (udev_log_priority) {
+               char priority[32];
+
+               sprintf(priority, "%i", udev_log_priority);
+               setenv("UDEV_LOG", priority, 1);
+       }
+
+       if (udev.type == DEV_BLOCK || udev.type == DEV_CLASS || udev.type == DEV_NET) {
+               udev_rules_init();
 
-       if (udev.type == BLOCK || udev.type == CLASS || udev.type == NET) {
                if (strcmp(action, "add") == 0) {
                        /* wait for sysfs and possibly add node */
                        dbg("udev add");
 
                        /* skip subsystems without "dev", but handle net devices */
-                       if (udev.type != NET && subsystem_expect_no_dev(udev.subsystem)) {
+                       if (udev.type != DEV_NET && subsystem_expect_no_dev(udev.subsystem)) {
                                dbg("don't care about '%s' devices", udev.subsystem);
                                goto hotplug;
                        }
 
-                       snprintf(path, SYSFS_PATH_MAX, "%s%s", sysfs_path, udev.devpath);
+                       snprintf(path, sizeof(path), "%s%s", sysfs_path, udev.devpath);
+                       path[sizeof(path)-1] = '\0';
                        class_dev = wait_class_device_open(path);
                        if (class_dev == NULL) {
-                               dbg ("open class device failed");
+                               dbg("open class device failed");
                                goto hotplug;
                        }
                        dbg("opened class_dev->name='%s'", class_dev->name);
 
                        wait_for_class_device(class_dev, &error);
 
-                       /* init rules */
-                       namedev_init();
-
                        /* name, create node, store in db */
                        retval = udev_add_device(&udev, class_dev);
 
@@ -202,22 +198,34 @@ int main(int argc, char *argv[], char *envp[])
                                goto hotplug;
                        }
 
+                       udev_rules_get_run(&udev);
+                       if (udev.ignore_device) {
+                               dbg("device event will be ignored");
+                               goto hotplug;
+                       }
+
                        /* get node from db, remove db-entry, delete created node */
                        retval = udev_remove_device(&udev);
                }
 
-               /* run dev.d/ scripts if we created/deleted a node or changed a netif name */
-               if (udev.devname[0] != '\0') {
+               if (udev.devname[0] != '\0')
                        setenv("DEVNAME", udev.devname, 1);
-                       if (udev_dev_d)
-                               udev_multiplex_directory(&udev, DEVD_DIR, DEVD_SUFFIX);
+
+               if (udev_run && !list_empty(&udev.run_list)) {
+                       struct name_entry *name_loop;
+
+                       dbg("executing run list");
+                       list_for_each_entry(name_loop, &udev.run_list, node)
+                               execute_command(name_loop->name, udev.subsystem);
                }
-       } else if (udev.type == PHYSDEV) {
+
+       } else if (udev.type == DEV_DEVICE) {
                if (strcmp(action, "add") == 0) {
                        /* wait for sysfs */
                        dbg("devices add");
 
-                       snprintf(path, SYSFS_PATH_MAX, "%s%s", sysfs_path, devpath);
+                       snprintf(path, sizeof(path), "%s%s", sysfs_path, devpath);
+                       path[sizeof(path)-1] = '\0';
                        devices_dev = wait_devices_device_open(path);
                        if (!devices_dev) {
                                dbg("devices device unavailable (probably remove has beaten us)");
@@ -239,7 +247,8 @@ hotplug:
        if (udev_hotplug_d && managed_event)
                udev_multiplex_directory(&udev, HOTPLUGD_DIR, HOTPLUG_SUFFIX);
 
-exit:
+       udev_cleanup_device(&udev);
+
        logging_close();
        return retval;
 }
@@
 #endif
 
-/* decide if we should manage the whole hotplug event
- * for now look if the kernel calls udevsend instead of /sbin/hotplug
+/* Decide if we should manage the whole uevent, including multiplexing
+ * of the hotplug directories.
+ * For now look if the kernel calls udevsend instead of /sbin/hotplug,
+ * or the uevent-helper in /proc/sys/kernel/hotplug is empty.
  */
 static int manage_hotplug_event(void) {